/*************************************************

 Statistical Analysis with Missing Data:

 

 K. Welch, B. Gillespie, and B. West



 Multiple Imputation using SAS Software.

*************************************************/



title1 "Missing Data Workshop. Lab Session 1.";

options linesize=80 pagesize=72 pageno=1 nodate label;



libname lab "C:\Documents and Settings\kwelch\Desktop\lsoa";



/*****************************************************************

 First, take a look at the values of the original variables

******************************************************************/

proc contents data=lab.lsoa_86x;

title2 "LSOA Data Set";

run;



proc means data=lab.lsoa_86x n nmiss mean std;

var numadl numadl2r poverty ownbuyr mortgage amtowed presval

    educ age84 racer sex ;

title2 "Descriptives for Variables in Original Data Set";

run;



proc freq data=lab.lsoa_86x;

  tables ownbuyr mortgage / missing;

  tables racer / missing;

  tables numadl numadl2r / missing;

  tables educ / missing;

title2 "Frequencies of Original Variables";

run;



/******************************************************************

   Now, read the permanent LSOA data set with missing data

   into a temporary working data set and create new variables.

   Note: different types of missing values can be indicated by

   using either a simple . or a code such as .D

*******************************************************************/



data one;

   set lab.lsoa_86x;



/* Recode the not-applicables to differentiate from missing */

   if ownbuyr=. then ownbuyr=5;

   if mortgage=. then mortgage=4;

   if ownbuyr=5 then amtowed=0;

   if ownbuyr=5 then presval=0;



/* Create a binary ownership variable. */

   own=0;

   if ownbuyr in (1,2) then own=1;

   if ownbuyr=.D then own=.;



/* Take natural log transforms of number of ADLs requiring assistance

   in 1986 and 1984 (to improve normality), and compute the difference.

   Add 1 to avoid taking logarithm of zero.

   NUMADL= Number of restricted activities of daily living (baseline, 1984)

   NUMADL2R= Number of restricted activities of daily living (followup, 1986)

*/

   logadl2r=log(numadl2r+1);

   logadl=log(numadl+1);

   diffadl=logadl2r-logadl;



/* Create dummy variables for race */



   black=0;

   if racer=2 then black=1;

   if racer=. then black=.;

   other=0;

   if racer=3 then other=1;

   if racer=. then other=.;

run;



/******************************************************************

   Now get the number of missing values and the distributions

   of respondent values for the variables of interest.

********************************************************************/



proc means data = one n nmiss mean std;

   var diffadl poverty own educ  age84 sex black other;

   title2 "Amount of Missing data and the Distributional";

   title3 "Characteristics of Respondent Values";

run;





/*********************************************************/

/* Analysis Method #1: Complete Case Regression Analysis */

/*********************************************************/



/* Fit a regression model predicting diffadl with a variety of

   demographic predictors */



proc reg data = one;

   model diffadl=poverty own educ logadl age84 sex black other;

   output out=regdat rstudent=resid;

   title2 "Regression Analysis: Complete Case Method. The Default.";

run;

quit;

proc univariate data=regdat plot; 

  var resid; 

  histogram;

  qqplot / normal (mu=est sigma=est);

run;



/* Note that 1883 observations are excluded from the analysis!

There are some variables with very few missing values.

Common Approach: Substitute mean or modal values. Theoretically,

this is problematic. We will assess the impact of doing this naive

imputation later. */



/*******************************************************************/

/* Analysis Method #2: Multiple Imputation Analysis using

   SAS Procs MI and MIANALYZE, Assuming Multivariate Normality

   for the variables to be imputed. */

/*******************************************************************/



/* We now use Proc MI to impute all of the missing data in the very

   first data set (one).

   Proc MI has different options for monotone and non-monotone 

   missing data.  Our example data have a non-monotone missing data 

   pattern, so we use the full-data Markov-Chain Monte Carlo (MCMC) 

   method. 



   NOTE:  Proc MI assumes a multivariate normal distribution for all 

   variables to be imputed.

   The variable OWN is binary, and therefore clearly non-normal.

   For now, we ignore this problem, and continue as if all variables 

   were normally distributed. */



/* Create a new data set, keeping only the key analysis variables

   from the original data set. Make sure all missing value

   codes are only . (not .D), as required by Proc MI. */



data five;

   set one;

   keep diffadl poverty own educ logadl logadl2r age84 sex racer black other ownmiss;

    /* Proc MI only likes . as missing values */

   if poverty = .D then poverty = .;

   if educ = .D then educ = .;



  /* Create a dummy variable for missing on OWN, to be used later. */

   if own = . then ownmiss = 1;

   if own in (0,1) then ownmiss = 0;



   /* drop cases that are missing on all continuous variables */

   if diffadl = . and poverty = . and educ = . and logadl = . then delete;



run;

/* First run Proc MI to look at the missing data patterns and check 

   for monotonicity. */

ods trace on;

proc mi data=five nimpute=0 simple ;

   var diffadl poverty own educ logadl;

   ods exclude  EMPostEstimates;

   title2 "Proc MI Analysis to Get Missing Data Patterns";

run;

ods trace off;



/* Run Proc MI to impute values for the variables with missing data. */



/* Note: 5 imputed data sets will be generated by default.

   Seed is a random number seed, for MCMC.

   The 5 imputed data sets will be saved in the data set outmi,

   and are indexed by the variable _Imputation_ */



proc mi data=five nimpute=5 out=outmi seed=3237851;

   var diffadl poverty own educ logadl ;

   title2 "Proc MI Analysis Using Multivariate Normal Assumption";

run;



/* Look at the new distributions of the imputed analysis variables

   for the first imputed data set.

   Note that the minimum and maximum imputed values for the binary

   variable OWN are outside the range of 0 to 1, because we assumed 

   multivariate normality.  */

options nolabel;

proc means data = outmi n nmiss  min max;

   class _Imputation_ ;

   var  own  logadl;

   title2 "Distributional Characteristics of Respondent Values";

   title3 "After Proc MI";

run;



/* Now, fit the regression model by _Imputation_

   Get separate parameter estimates for each imputed data set,

   and save the resulting estimates in a data set named outreg.

   Note: the noprint option means that the regression output

   for each imputed data set will not show up in the SAS output

   window. */



proc reg data=outmi outest=outreg covout noprint;

   by _Imputation_;

   model diffadl=poverty own educ logadl age84 sex black other;

run;



/* Now, use Proc MIANALYZE to perform a multiple imputation analysis

   using the estimated coefficients and standard errors from

   each of the imputed data sets. */



proc mianalyze data=outreg;

   modeleffects Intercept poverty own educ logadl age84 sex black other;

   title2 "Combine the Estimates Using Proc MIANALYZE";

run;



/*******************************************************************

   Analysis Method #3:

   Alternative Method for Imputation for the Binary Variable: OWN.

   Using the Imputed data sets, we now set the imputed values of OWN

   to missing.



   Then, we rerun Proc MI on each of the 5 imputed data sets, which

   are now monotone missing for OWN, and generate a _single_ imputed

   data set for each imputed data set using the Logistic Method.

   NOTE: The logistic regression method is considered EXPERIMENTAL by SAS.



   Then, we rerun Proc Reg and Proc MIANALYZE.

**********************************************************************/



/* Reset the missing values of OWN to missing in the imputed data set*/

data six;

  set outmi;

  if ownmiss = 1 then own = .;

run;



proc mi data=six nimpute=1 out=outmi2 seed=3237851;

   by _Imputation_;

   class own;

   monotone logistic (OWN = poverty educ logadl age84 sex black other / details);

   var poverty educ logadl age84 sex black other own;

   title2 "Impute Values for OWN, Using Logistic regression on the Monotone Data";

run;



proc freq data=outmi2;

  tables _imputation_*own / nocol nopercent;

  title2 "Distribution of Imputed Values After Logistic Regression";

  title3 "Imputed Values for OWN are now 0 or 1";

run;



proc reg data=outmi2  outest=outreg2 covout noprint;

  by _imputation_;

  model diffadl=poverty own educ logadl age84 sex black other;

run;



/* Now, use Proc MIANALYZE to perform a multiple imputation analysis

   using the estimated coefficients and standard errors from

   each of the imputed data sets. */



proc mianalyze data=outreg2 ;

   modeleffects Intercept poverty own educ logadl age84 sex black other;

   title2 "Multiple Imputation Analysis After Logistic Regression for OWN";

run;









/************************************************************

  Redo Multiple Imputation With Min and Max Set,

  Note: Make all missing values . Prior to running Proc MI

*************************************************************/

proc mi data=FIVE nimpute=15 out=OUTMI3 seed=3355

       minimum = 1 0 . . . 0  . 0 0

       maximum = 2 1 . . . 18 . . .;

   var poverty own sex black other educ age84 logadl logadl2r ;

   title2 "Multiple Imputation Using Multivariate Normal Assumption";

   title3 "With Min and Max Specified";

run;



data OUTMI3;

  set OUTMI3;

  diffadl = logadl2r - logadl;

run;



options nolabel;

proc means data=OUTMI3 n nmiss min max mean;

  by _Imputation_;

  title2 "Descriptives on Imputed Data Sets, After Setting Min and Max";

run;



proc reg data=OUTMI3 outest=OUTREG3 covout ;

   by _Imputation_;

   model diffadl=poverty own educ logadl age84 sex black other;

   title2 "Regression Analysis on Imputed Data";

run;



proc mianalyze data=OUTREG3;

   modeleffects Intercept poverty own educ logadl age84 sex black other;

   title2 "Combine the Estimates Using Proc MIANALYZE";

   title3 "Assume Multivariate Normal Distribution, but with Min and Max";

run;



/****************************************************************

 MIANALYZE on Parameter Estimates and Covariances from GENMOD

*****************************************************************/

proc genmod data=outmi;

  model diffadl = poverty own educ logadl age84 sex black other / covb dist=normal;

  by _Imputation_;

  ods output ParameterEstimates=gmparms 

             ParmInfo=gmpinfo

             CovB=gmcovb;

  title2 "Proc GENMOD on Imputed Data";

run;



proc print data=gmparms(obs=20);

  var _Imputation_ Parameter Estimate StdErr;

  title2 "GENMOD Model Coefficients (First Two Imputations)";

run;



proc print data=gmpinfo (obs=20);

  title2 "GENMOD Parameter Information (First Two Imputations)"; 

run;



proc print data=gmcovb(obs=20);

  var _Imputation_ RowName Prm1-Prm9;

  title2 "GENMOD Covariance Matrices (First Two Imputations)";

run;



proc mianalyze parms=gmparms covb=gmcovb parminfo=gmpinfo;

  modeleffects intercept poverty own educ logadl age84 sex black other;

  test Poverty = Own = Educ = 0 / mult;

  title2 "Analysis of Coefficients from GENMOD";

run;

   



/****************************************************************

 MIANALYZE on Parameter Estimates and Covariances from LOGISTIC

*****************************************************************/

data outmi2;

  set outmi2;

  /*Set up imputed values to be integers*/

  if poverty < 1.5 then poverty2 = 1;

  if poverty >=1.5 then poverty2 = 2;

run;

proc freq data=outmi2;

  tables poverty2;

run;



proc logistic data=outmi2;

  model poverty2 = own educ own*educ age84 sex black other / covb ;

  by _Imputation_;

  ods output ParameterEstimates=lgparms

             CovB=lgcovb;

  title2 "Proc LOGISTIC on Imputed Data";

run;

proc print data=lgparms (obs=16);

  title2 "LOGISTIC Model Coefficients (First Two Imputations)";

run;

proc print data=lgcovb (obs=16);

  title2 "LOGISTIC Model Covariance Matrices (First Two Imputations)";

run;

proc mianalyze parms=lgparms

                  covb(effectvar=stacking)=lgcovb;

   modeleffects Intercept own educ own*educ age84 sex black other;

   title2 "Analysis of Coefficients from LOGISTIC";

run;



/****************************************************************

 MIANALYZE on Parameter Estimates and Covariances from MIXED

*****************************************************************/

proc mixed data=outmi;

  model diffadl=poverty own educ logadl age84 sex black other/solution covb;

  by _Imputation_;

  ods output SolutionF=mixparms CovB=mixcovb;

  title2 "Use Proc Mixed to Analyze Imputed Data";

run;

proc print data=mixparms (obs=18);

  var _Imputation_ Effect Estimate StdErr;

  title "Model Coefficients from Proc MIXED (First Two Imputations)";

run;

proc print data=mixcovb (obs=18);

 var _Imputation_ Row Effect Col1-Col9;

 title "Covariance Matrices (First Two Imputations)";

run;

proc mianalyze parms=mixparms 

  covb(effectvar=rowcol)=mixcovb;

  modeleffects Intercept poverty own educ logadl age84 sex black other ;

  SES: test Poverty = OWN = EDUC = 0 / mult;

  RACE: test black = 0, other = 0 / mult;

  title "Analysis of Coefficients from Proc MIXED";

run;



/****************************************************************

 MIANALYZE on Parameter Estimates and Covariances from MIXED

  with a CLASS Statement

*****************************************************************/

proc means data=outmi;

run;

proc mixed data=outmi;

  class racer;

  model diffadl=poverty own educ logadl age84 sex racer/solution covb;

  by _Imputation_;

  ods output SolutionF=mixparms2;

  title2 "Use Proc Mixed to Analyze Imputed Data";

  title2 "Including a CLASS statement for RACER";

run;

proc print data=mixparms2 (obs=20);

  var _Imputation_ Effect RACER Estimate StdErr;

  title "Model Coefficients from Proc MIXED (First Two Imputations)";

run;

proc mianalyze parms(classvar=full)=mixparms2;

  class racer;

  modeleffects Intercept poverty own educ logadl age84 sex racer;

  title "Analysis of Coefficients from Proc MIXED, with CLASS Variable";

run;



/****************************************************************

 MIANALYZE on a *Single* z-Transformed Correlation from CORR

*****************************************************************/

title;

proc corr data=outmi fisher(biasadj=no);

 var logadl logadl2r ;

 by _Imputation_;

 ods output FisherPearsonCorr= outz;

run;



proc print data=outz;

 title "Fisher's Correlation Statistics";

 var _Imputation_ NObs ZVal;

run;



data outz;

      set outz;

      StdZ= 1. / sqrt(NObs-3);

run;







/*The following statements use the MIANALYZE procedure to generate 

  a combined parameter estimate  and its variance. 

  The ODS statement is used to save the parameter estimates 

  in an output data set. */



proc mianalyze data=outz;

      ods output ParameterEstimates=parms;

      modeleffects ZVal;

      stderr StdZ;

run;



proc print data=parms;

      title 'Parameter Estimates with 95% Confidence Limits';

      var Estimate LCLMean UCLMean; 

run;

data corr_ci;

      set parms;

      r=       tanh( Estimate);

      r_lower= tanh( LCLMean);

      r_upper= tanh( UCLMean);

   run;



proc print data=corr_ci;

      title "Estimated Correlation Coefficient with 95% Confidence Limits";

      var r r_lower r_upper;

run;



proc means data=one;

run;



